﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

[RequireComponent(typeof(CharacterController2D), typeof(Animator))]
public abstract class Actor : MonoBehaviour
{
	//--- MEMBERS -----------------------------------------------------------------------------------------

    public MovementParameters movementParameters;

    public float Gravity { get { return movementParameters.gravity; } }
    public float MovementSpeed { get { return movementParameters.movementSpeed; } }
    public float AccelerationGround { get { return movementParameters.accelerationGround; } }
    public float AccelerationAir { get { return movementParameters.accelerationAir; } }
    public float JumpForce { get { return movementParameters.jumpForce; } }

    public CharacterController2D.CollisionData CollisionData { get { return _controller.collisionData; } }
    protected Interactable CollidingObject { get { return CollisionData.collidingObject; } }

    protected CharacterController2D _controller;
    protected Vector2 _velocitySmoothing;
    protected Vector2 _velocity;

	protected Animator _animator;

    [SerializeField]
    protected int _pushStrength = 1;
    protected float _pushSpeed = 3f;

    protected float _summonTime = .6f;
    protected float _summonTimeLeft;

    internal Actor spiritBond;

    //---- METHODS ----------------------------------------------------------------------------------------

    protected virtual void Awake()
    {
        _controller = GetComponent<CharacterController2D>();
		_animator = GetComponent<Animator>();
        _summonTimeLeft = _summonTime;
    }

    //-----------------------------------------------------------------------------------------------------

    protected void PushObject(Vector2 direction)
    {
        CollidingObject.Move(direction * _pushSpeed * Time.deltaTime);
        _controller.Move(new Vector2(1 * direction.x * _pushSpeed, -.15f) * Time.deltaTime);
    }

    //-----------------------------------------------------------------------------------------------------
}