﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class Altar : MonoBehaviour
{
    private PuzzlePiece[] _puzzle;
    public Actor soulAnimalReward;

    private CaveManActor _caveman;
    private int _selectedPieceIndex;
    private bool _isPuzzleSolved = false;
    private Transform _pieceSelector;
    private float _selectorWaitTime = 0.2f;
    private float _selectorRemainingWaitTime = 0f;

    //----------------------------------------------------------------------------------------------------

    private void Awake()
    {
        _pieceSelector = transform.Find("PieceSelector");
        _puzzle = transform.GetComponentsInChildren<PuzzlePiece>();

		transform.GetChild (0).GetComponent<Light> ().enabled = false;

        foreach (PuzzlePiece piece in _puzzle)
        {
            piece.Scramble();
        }
    }

    //----------------------------------------------------------------------------------------------------

    private void RevealPuzzle()
    {
        _selectedPieceIndex = 0;

        StartCoroutine(LetPlayerSolve());
    }

    //-----------------------------------------------------------------------------------------------------

    private IEnumerator LetPlayerSolve()
    {
        while (_isPuzzleSolved == false)
        {
            HandleInput();
            yield return null;
        }

      //  print("You solved it!");
		transform.GetChild (0).GetComponent<Light> ().enabled = false;
		transform.GetComponent<Animator> ().SetBool ("InUse", false);
		_caveman.GetComponent<Animator> ().SetBool ("Summon", false);
        _caveman.enabled = true;
        _caveman.spiritAnimal = soulAnimalReward;
    }

    //-----------------------------------------------------------------------------------------------------

    private void ChangeIndex(int input)
    {
        int newIndex = _selectedPieceIndex + input;
        
		if(newIndex <= _puzzle.Length-1 && newIndex >= 0)
        {
            _selectedPieceIndex = newIndex;
            return;
        }

        if(input > 0)        
            _selectedPieceIndex = newIndex - _puzzle.Length;
        
        if(input < 0)        
            _selectedPieceIndex = _puzzle.Length + newIndex;
                
    }

    //-----------------------------------------------------------------------------------------------------

    private void HandleInput()
    {
        _selectorRemainingWaitTime -= Time.deltaTime;

        if(_selectorRemainingWaitTime <= 0)
        {
            Vector2 input = new Vector2(Input.GetAxisRaw("Horizontal"), Input.GetAxisRaw("Vertical"));
            ChangeIndex((int)input.x);
            ChangeIndex((int)input.y * -3);

			_pieceSelector.transform.position = _puzzle [_selectedPieceIndex].transform.position + Vector3.back * 1.5f;
            _selectorRemainingWaitTime = _selectorWaitTime;
        }

        if (Input.GetButtonDown("Jump"))
        {
            _puzzle[_selectedPieceIndex].Rotate();

            // If a puzzle piece has been aligned correctly, check the current state of the puzzle
            if (_puzzle[_selectedPieceIndex].rotationFactor == 0)
            {
                _isPuzzleSolved = CheckPuzzleStatus();
            }
        }
    }

    //-----------------------------------------------------------------------------------------------------

    public bool CheckPuzzleStatus()
    {
        foreach (PuzzlePiece piece in _puzzle)
        {
            if (piece.rotationFactor != 0)
                return false;
        }

        return true;
    }

    //-----------------------------------------------------------------------------------------------------

    private void OnTriggerEnter2D(Collider2D other)
    {
        if (other.CompareTag("Caveman"))
        {
			transform.GetChild (0).GetComponent<Light> ().enabled = true;
			transform.GetComponent<Animator> ().SetBool ("InUse", true);
            _caveman = other.GetComponent<CaveManActor>();
			_caveman.enabled = false;
			_caveman.GetComponent<Animator> ().SetBool ("Walking", false);
			_caveman.GetComponent<Animator> ().SetBool ("Summon", true);
            RevealPuzzle();
        }
    }

    //-----------------------------------------------------------------------------------------------------
}