﻿using UnityEngine;
using System.Collections;

[RequireComponent(typeof(Camera))]
public class CameraController : MonoBehaviour 
{
	//--- MEMBERS -----------------------------------------------------------------------------------------

	[HideInInspector] public Transform target;
	[SerializeField, Range(16, 400)] private int _xBoundary = 100;
	[SerializeField, Range(0f, 16f)] private float _targetMinX, _targetMaxX;

	//[SerializeField, Range(0f, 8f)] private float _xOffset = 4;
	//[SerializeField, Range(0f, 10f)] private float _cameraSpeed;

	private float _cameraWidth;

	//---- METHODS ----------------------------------------------------------------------------------------

	private void Awake()
	{
		Camera camera = GetComponent<Camera>(); 
		_cameraWidth = Mathf.Round(2 * camera.aspect * camera.orthographicSize);
	}

	//-----------------------------------------------------------------------------------------------------

	private void Update() 
	{
		if(target == null)
			return;

		float targetRelativeX = target.position.x - transform.position.x + .5f * _cameraWidth;

		float minX = _targetMinX;
		float maxX = _targetMaxX;
		if(target.transform.localScale.x == -1f)
		{
			minX = _cameraWidth - _targetMaxX;
			maxX = _cameraWidth -  _targetMinX;
		}

		if(targetRelativeX < minX  || targetRelativeX > maxX)
		{
			//float cameraTargetX = Mathf.Clamp(transform.position.x + _cameraSpeed * Time.deltaTime * (targetRelativeX < minX ? -1f : 1f), .5f *  _cameraWidth, _xBoundary - .5f * _cameraWidth);
			float cameraTargetX = Mathf.Clamp(transform.position.x + (targetRelativeX < minX ? minX - targetRelativeX : targetRelativeX - maxX) * (targetRelativeX < minX ? -1f : 1f), .5f *  _cameraWidth, _xBoundary - .5f * _cameraWidth);
			//print("Moving Camera to " + cameraTargetX);
			transform.position = new Vector3(cameraTargetX, transform.position.y, transform.position.z);
		}

		//Vector3 targetPosition = new Vector3(Mathf.Clamp(target.position.x + _xOffset, 0f + .5f * _cameraWidth, _xBoundary - .5f * _cameraWidth), transform.position.y,  transform.position.z);
		//transform.position = Vector3.MoveTowards(transform.position, targetPosition, _cameraSpeed * Time.deltaTime);
	}

	//-----------------------------------------------------------------------------------------------------
}