﻿using UnityEngine;
using System.Collections;

public class CaveManActor : Actor
{
    //--- MEMBERS -----------------------------------------------------------------------------------------

    public Actor spiritAnimal;

    public bool IsTorchEquipped { get { return _torch.activeSelf; } }

    private GameObject _torch;
    private Transform _holsterSearchEmitter;

    [SerializeField]
    private LayerMask _holsterMask;

	private bool _canHolster;

    //---- METHODS ----------------------------------------------------------------------------------------

    protected override void Awake()
    {
        base.Awake();
        Camera.main.GetComponent<CameraController>().target = transform;
        _torch = transform.FindChild("Torch").gameObject;
        _holsterSearchEmitter = transform.FindChild("HolsterSearch");
        _summonTimeLeft = _summonTime;
		_canHolster = true;
    }

    //-----------------------------------------------------------------------------------------------------

    private void Update()
    {
        HandleInput();

        _velocity.y += Gravity * Time.deltaTime;
        _controller.Move(_velocity * Time.deltaTime);

        if (CollisionData.top || CollisionData.bottom)
        {
            _velocity.y = 0;
        }

        if (CollisionData.left || CollisionData.right)
        {
            float dirX = Mathf.Sign(_velocity.x);
            _velocity.x = 0;
            _velocitySmoothing.x = 0;

            if (CollidingObject != null && CollidingObject.weight <= _pushStrength)
            {
                PushObject(Vector2.right * dirX);
            }
        }

        if (Input.GetButtonDown("Pause"))
        {
            GameManager.instance.PauseGame();
        }

        if (transform.position.y < -3)
        {
            GameManager.instance.KillPlayer(GameManager.PlayerKillMethod.Pit);
            Destroy(gameObject);
        }
    }

    //-----------------------------------------------------------------------------------------------------

    private void HandleInput()
    {
        Vector2 input = new Vector2(Input.GetAxisRaw("Horizontal"), Input.GetAxisRaw("Vertical"));
        _velocity.x = Mathf.SmoothDamp(_velocity.x, input.x * MovementSpeed, ref _velocitySmoothing.x, CollisionData.bottom ? AccelerationGround : AccelerationAir);

        if (input.x != 0f)
            transform.localScale = new Vector3(Mathf.Sign(input.x), transform.localScale.y, transform.localScale.z);
        _animator.SetBool("Walking", input.x != 0f);

        if (Input.GetButtonDown("Jump"))
        {
            if (CollisionData.bottom)
                _velocity.y = JumpForce;
        }

		if(input.y <= 0)
			_canHolster = true;

        if (input.y != -1)
            _summonTimeLeft = _summonTime;


		if (input.y == 1 && input.x == 0 && _canHolster)
        {
            DetectHolsters();
			_canHolster = false;
        }
        else if (input.y == -1 && input.x == 0 && CollisionData.bottom)
        {
            _summonTimeLeft -= Time.deltaTime;

            if(_summonTimeLeft <= 0)
            {
                SummonAnimal();
            }
        }
    }

    //-----------------------------------------------------------------------------------------------------

    public void DetectHolsters()
    {
        Vector2 rayOrigin = _holsterSearchEmitter.position;
        float directionX = Mathf.Sign(transform.localScale.x);

        RaycastHit2D hit = Physics2D.Raycast(rayOrigin, Vector2.right * directionX, 0.7f, _holsterMask);

        Debug.DrawRay(rayOrigin, Vector2.right * directionX * 0.7f, Color.red);

        if(hit)
        {
            if (IsTorchEquipped)
            {
                // Uba Store Animation
                _torch.SetActive(false);
                hit.transform.FindChild("Torch").gameObject.SetActive(true);
				_animator.SetBool("HasTorch", false);
            }
            else
            {
                GameObject holsterTorch = hit.transform.FindChild("Torch").gameObject;

                if (holsterTorch)
                {
                    // Uba Pickup Animation
                    holsterTorch.SetActive(false);
                    _torch.SetActive(true);
					_animator.SetBool("HasTorch", true);
                }
            }

        }
    }

    //-----------------------------------------------------------------------------------------------------

    private void SummonAnimal()
    {
		if(spiritAnimal == null)
			return;
		
		_animator.SetBool("Summon", true);
        spiritBond = Instantiate(spiritAnimal, transform.position, transform.rotation) as Actor;
        spiritBond.spiritBond = this;
		Camera.main.GetComponent<CameraController>().target = spiritBond.transform;
        _summonTimeLeft = _summonTime;
        enabled = false;
    }

    //-----------------------------------------------------------------------------------------------------


    public void BlowOutTorch()
    {
        // Take care of blown out torch animation;
    }

    //-----------------------------------------------------------------------------------------------------

}