﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;


public class CharacterController2D : RaycastingObject
{
    public struct CollisionData
    {
        public bool left, right, top, bottom;
        public Interactable collidingObject;

        public void Reset()
        {
            left = right = top = bottom = false;
        }
    }

    public CollisionData collisionData;

    private Transform _transform;

    // ----------------------------------------------------------------------------------------

    protected override void Awake()
    {
        base.Awake();

        _transform = transform;
    }

    // ----------------------------------------------------------------------------------------

    public void Move(Vector2 velocity)
    {
        collisionData.Reset();
        UpdateRayOrigins();        

        if (velocity.x != 0)
            CheckHorizontalCollisions(ref velocity);

        if (velocity.y != 0)
            CheckVerticalCollisions(ref velocity);

        _transform.Translate(velocity);
    }

    // ----------------------------------------------------------------------------------------

    private void CheckHorizontalCollisions(ref Vector2 velocity)
    {
        int directionX = velocity.x > 0 ? 1 : -1;
        float rayLength = Mathf.Abs(velocity.x) + SkinWidth;

        for(uint i = 0; i < rayCount.x; ++i)
        {
            Vector2 rayOrigin = directionX == 1 ? rayOrigins.topRight : rayOrigins.topLeft;
            rayOrigin.y -= raySpacing.x * i;

            RaycastHit2D hit = Physics2D.Raycast(rayOrigin, Vector2.right * directionX, rayLength, _collisionMask);
            //Debug.DrawRay(rayOrigin, Vector2.right * directionX * rayLength, Color.red);

            if (hit)
            {
                rayLength = hit.distance - SkinWidth;
                velocity.x = directionX == 1 ? rayLength : -rayLength;

                collisionData.right = directionX == 1;
                collisionData.left = directionX == -1;

                collisionData.collidingObject = hit.transform.GetComponent<Interactable>();
            }
        }
    }

    // ----------------------------------------------------------------------------------------

    private void CheckVerticalCollisions(ref Vector2 velocity)
    {
        int directionY = velocity.y > 0 ? 1 : -1;
        float rayLength = Mathf.Abs(velocity.y) + SkinWidth;

        for (uint i = 0; i < rayCount.y; ++i)
        {
            Vector2 rayOrigin = directionY == 1 ? rayOrigins.topLeft : rayOrigins.bottomLeft;
            rayOrigin.x += raySpacing.y * i + velocity.x;

            RaycastHit2D hit = Physics2D.Raycast(rayOrigin, Vector2.up * directionY, rayLength, _collisionMask);
            //Debug.DrawRay(rayOrigin, Vector2.up * directionY * rayLength, Color.red);

            if (hit)
            {
                rayLength = hit.distance - SkinWidth;
                velocity.y = directionY == 1 ? rayLength : -rayLength;

                collisionData.top = directionY == 1;
                collisionData.bottom = directionY == -1;
            }
        }
    }

    // ----------------------------------------------------------------------------------------

    public bool FireRays(Vector2 direction, float rayLength = SkinWidth *2)
    {
        for (uint i = 0; i < rayCount.x; ++i)
        {
            Vector2 rayOrigin = direction.x == 1 ? rayOrigins.topRight : rayOrigins.topLeft;
            rayOrigin.y -= raySpacing.x * i;

            RaycastHit2D hit = Physics2D.Raycast(rayOrigin, direction, rayLength, _collisionMask);

            if (hit)
            {
                return true;
            }
        }

        return false;
    }

    // ----------------------------------------------------------------------------------------

}

