﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class FoxActor : Actor
{
    public Vector2 WallJumpForce { get { return movementParameters.wallJumpForce; } }

    public CaveManActor caveMan;

    public float wallStickTime = .3f;
    private bool _isSticking = false;

    private Coroutine _routine;

    // ----------------------------------------------------------------------------------------

    protected override void Awake()
    {
        base.Awake();
        _summonTimeLeft = _summonTime;
    }

    // ----------------------------------------------------------------------------------------

    private void Update()
    {
		// Destroy fox when falling out of the map
		if(transform.position.y <  -3f)
			RemoveSelf();

        HandleInput();

        _velocity.y += Gravity * Time.deltaTime;
        _controller.Move(_velocity * Time.deltaTime);

        if (CollisionData.top || CollisionData.bottom)
        {
            _velocity.y = 0;
            _isSticking = false;
            if (_routine != null)
                StopCoroutine(_routine);

			if(CollisionData.bottom)
			{
				if(_animator.GetBool("Grounded") == false)
					_animator.SetBool("Grounded", true);
			}
        }

		if(CollisionData.bottom == false)
			_animator.SetBool("Grounded", false);


        if (CollisionData.left || CollisionData.right)
        {
            float dirX = Mathf.Sign(_velocity.x);
            _velocity.x = 0;
            _velocitySmoothing.x = 0;

            if(CollidingObject != null && CollidingObject.weight <= _pushStrength)
            {
                PushObject(Vector2.right * dirX);
            }

            if (CollisionData.bottom == false)
            {
                if (_isSticking == false)
                    _routine = StartCoroutine(Unstick());
            }
        }
    }

    // ----------------------------------------------------------------------------------------

    private IEnumerator Unstick()
    {
        _isSticking = true;
        yield return new WaitForSeconds(wallStickTime);
        _isSticking = false;
    }

    // ----------------------------------------------------------------------------------------

    private void HandleInput()
    {
        Vector2 input = new Vector2(Input.GetAxisRaw("Horizontal"), Input.GetAxisRaw("Vertical"));
        float targetVelX = Mathf.SmoothDamp(_velocity.x, input.x * MovementSpeed, ref _velocitySmoothing.x, CollisionData.bottom ? AccelerationGround : AccelerationAir);

		if (input.x != 0f)
			transform.localScale = new Vector3(Mathf.Sign(input.x), transform.localScale.y, transform.localScale.z);
		_animator.SetBool("Walking", input.x != 0f);

        if (input.y == -1 && input.x == 0 && CollisionData.bottom)
        {
            _summonTimeLeft -= Time.deltaTime;

            if (_summonTimeLeft <= 0)
            {
				RemoveSelf();
            }
        }
        else
            _summonTimeLeft = _summonTime;

        if (_isSticking == false)
            _velocity.x = targetVelX;

        if (Input.GetButtonDown("Jump"))
        {
            if (CollisionData.bottom)
			{
				_velocity.y = JumpForce;
				_animator.SetTrigger("Jump");
			}              
            else
            {
                // Wall Jumping (kinda hacky)
                if (_controller.FireRays(Vector2.left))
                {
                    _velocity.x = WallJumpForce.x;
                    _velocity.y = WallJumpForce.y;
					_animator.SetTrigger("Walljump");
                }
                else if (_controller.FireRays(Vector2.right))
                {
                    _velocity.x = -WallJumpForce.x;
                    _velocity.y = WallJumpForce.y;
					_animator.SetTrigger("Walljump");
                }
            }
        }

        if (_controller.FireRays(Vector2.left) == false && _controller.FireRays(Vector2.right) == false)
        {
            _isSticking = false;
            if (_routine != null)
                StopCoroutine(_routine);
        }
    }

    // ----------------------------------------------------------------------------------------

	private void RemoveSelf()
	{
		spiritBond.enabled = true;
		Camera.main.GetComponent<CameraController>().target = spiritBond.transform;
		spiritBond.GetComponent<Animator>().SetBool("Summon", false);
		Destroy(gameObject);
	}

	// ----------------------------------------------------------------------------------------

}