﻿using UnityEngine;
using UnityEngine.SceneManagement;
using System.Collections;
using UnityEngine.UI;
using System.IO;

public class GameManager : MonoBehaviour
{

    //--- MEMBERS -----------------------------------------------------------------------------------------

    public static GameManager instance;
    public XmlManager xmlManager;
    public int savedCheckPoint;
    public bool canResume;
    [HideInInspector] public CaveManActor caveman;
	public Transform spawnPoints;
	private GameObject canvas;

    public enum PlayerKillMethod { Torch, Pit }

    //---- METHODS ----------------------------------------------------------------------------------------

    void Awake()
    {
        Cursor.visible = false;
        Cursor.lockState = CursorLockMode.Locked;

        if (instance == null)
            instance = this;
        else if (instance != this)
            Destroy(gameObject);

        DontDestroyOnLoad(gameObject);

		if (File.Exists (Path.Combine (Application.dataPath, "data.cp")) == false) 
		{
			xmlManager = new XmlManager ();
			xmlManager.Save ();
			//File.Create (Path.Combine(Application.persistentDataPath, "data.cp"));
		}
			

        xmlManager = XmlManager.Load();

        if ((xmlManager.scene > 0 || xmlManager.checkPoint > 0) && SceneManager.GetActiveScene().buildIndex == 0)
            canResume = true;

        caveman = FindObjectOfType<CaveManActor>();

		if(SceneManager.GetActiveScene().buildIndex > 0)
			canvas = GameObject.Find("Menu").transform.GetChild(0).gameObject;
    }

	//-----------------------------------------------------------------------------------------------------

	public void SpawnPlayer(int pointId)
	{
		
		caveman = GameObject.FindObjectOfType<CaveManActor>();
		spawnPoints = GameObject.Find ("SpawnPoints").transform;
		caveman.transform.position = spawnPoints.GetChild (pointId).position + Vector3.up * 1.55f;

		if (xmlManager.checkPoint == 0 && SceneManager.GetActiveScene().buildIndex != 1)
			StartCoroutine (PlayerAutoWalk (false));
	}

    //-----------------------------------------------------------------------------------------------------

    public void LoadScene(int sceneId)
    {
        SceneManager.LoadScene(sceneId);
    }

	//-----------------------------------------------------------------------------------------------------

	void OnLevelWasLoaded()
	{
		canvas = GameObject.Find("Menu").transform.GetChild(0).gameObject;
		canvas.transform.Find ("Buttons").GetChild (1).GetComponent<Button> ().onClick.AddListener (() => {
			RestartScene ();
		});
		canvas.transform.Find ("Buttons").GetChild (2).GetComponent<Button> ().onClick.AddListener (() => {
			Resume ();
		});
		canvas.transform.Find ("Buttons").GetChild (3).GetComponent<Button> ().onClick.AddListener (() => {
			QuitGame ();
		});
		savedCheckPoint = xmlManager.checkPoint;
		SpawnPlayer (savedCheckPoint);
	}

    //-----------------------------------------------------------------------------------------------------

    public void LoadSavePoint()
    {
        SceneManager.LoadScene(xmlManager.scene);
    }

    //-----------------------------------------------------------------------------------------------------

    public void KillPlayer(PlayerKillMethod method)
    {
		if (method == PlayerKillMethod.Pit) 
		{
			StartCoroutine(PitKill());
		}
		else if(method == PlayerKillMethod.Torch)
		{
			StartCoroutine(TorchKill());
		}
    }

    //-----------------------------------------------------------------------------------------------------

    public void RestartScene()
    {
        LoadScene(SceneManager.GetActiveScene().buildIndex);
    }

    //-----------------------------------------------------------------------------------------------------

    public void LoadNextScene()
    {
        LoadScene(SceneManager.GetActiveScene().buildIndex + 1);
		xmlManager.checkPoint = 0;
    }

    //-----------------------------------------------------------------------------------------------------

    public void SaveScene(int savedPoint)
    {
        xmlManager.checkPoint = savedPoint;
        xmlManager.scene = SceneManager.GetActiveScene().buildIndex;
        xmlManager.Save();
    }

	//-----------------------------------------------------------------------------------------------------

	public void PauseGame()
	{
		caveman.enabled = !caveman.enabled;
		canvas.gameObject.SetActive (!canvas.gameObject.activeSelf);
	}

	//-----------------------------------------------------------------------------------------------------

	public void Resume()
	{
		caveman.enabled = true;
		canvas.gameObject.SetActive (false);
	}

    //-----------------------------------------------------------------------------------------------------

    public void QuitGame()
    {
        SaveScene(SceneManager.GetActiveScene().buildIndex);
        Application.Quit();
    }

	//-----------------------------------------------------------------------------------------------------

	private IEnumerator PitKill()
	{
		//Spiel ein Audio
		yield return new WaitForSeconds (1.5f);
		RestartScene ();
	}

	//-----------------------------------------------------------------------------------------------------

	private IEnumerator TorchKill()
	{
		if(caveman.IsTorchEquipped == false)
			yield break;
		
		caveman.enabled = false;
		caveman.GetComponent<Animator>().SetBool("Walking", false);
		Transform torch = caveman.transform.GetChild(0);
		torch.GetComponent<Animator>().SetTrigger("Extinguish");
		torch.GetComponentInChildren<Light>().enabled  = false;
		yield return new WaitForSeconds(1.5f);
		RestartScene();
	}

	//-----------------------------------------------------------------------------------------------------

	public IEnumerator PlayerAutoWalk(bool loadNextScene)
	{
		caveman.enabled = false;

		Animator cavemanAnimator = caveman.GetComponent<Animator> ();

		cavemanAnimator.SetBool ("Walking", true);
		Vector3 target = caveman.transform.position + Vector3.right * 6f;

		while(caveman.transform.position != target)
		{
			caveman.transform.position = Vector3.MoveTowards (caveman.transform.position, target, 3f*Time.deltaTime);
			yield return null;
		}

		if (loadNextScene == true)
			LoadNextScene ();
		else
			caveman.enabled = true;
	}

	//-----------------------------------------------------------------------------------------------------
}
