﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

public class MammothActor : Actor
{
    protected override void Awake()
    {
        base.Awake();
    }

    // ----------------------------------------------------------------------------------------

    private void Update()
    {
        // Destroy MAMMOTH when falling out of the map
        if (transform.position.y < -3f)
            RemoveSelf();

        HandleInput();

        _velocity.y += Gravity * Time.deltaTime;
        _controller.Move(_velocity * Time.deltaTime);

        if (CollisionData.top || CollisionData.bottom)
        {
            _velocity.y = 0;
        }

        if (CollisionData.left || CollisionData.right)
        {
            float dirX = Mathf.Sign(_velocity.x);
            _velocity.x = 0;
            _velocitySmoothing.x = 0;

            if (CollidingObject != null && CollidingObject.weight <= _pushStrength)
            {
                PushObject(Vector2.right * dirX);
            }
        }
    }

    // ----------------------------------------------------------------------------------------

    private void HandleInput()
    {
        Vector2 input = new Vector2(Input.GetAxisRaw("Horizontal"), Input.GetAxisRaw("Vertical"));
        _velocity.x = Mathf.SmoothDamp(_velocity.x, input.x * MovementSpeed, ref _velocitySmoothing.x, CollisionData.bottom ? AccelerationGround : AccelerationAir);

        if (input.x != 0f)
            transform.localScale = new Vector3(Mathf.Sign(input.x), transform.localScale.y, transform.localScale.z);

        _animator.SetBool("Walking", input.x != 0f);

        if (input.y == -1 && input.x == 0 && CollisionData.bottom)
        {
            _summonTimeLeft -= Time.deltaTime;

            if (_summonTimeLeft <= 0)
            {
                RemoveSelf();
            }
        }
        else
            _summonTimeLeft = _summonTime;

        if (Input.GetButtonDown("Jump"))
        {
            if (CollisionData.bottom)
            {
                _velocity.y = JumpForce;
                _animator.SetTrigger("Jump");
            }
        }
    }

    // ----------------------------------------------------------------------------------------

    private void RemoveSelf()
    {
        spiritBond.enabled = true;
        Camera.main.GetComponent<CameraController>().target = spiritBond.transform;
        spiritBond.GetComponent<Animator>().SetBool("Summon", false);
        Destroy(gameObject);
    }

    // ----------------------------------------------------------------------------------------

}