﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

[RequireComponent(typeof(Rigidbody2D), typeof(BoxCollider2D))]
public abstract class RaycastingObject : MonoBehaviour
{
    public const float SkinWidth = .015f;

    [SerializeField]
    protected LayerMask _collisionMask;
    [SerializeField]
    protected Vector2 rayCount = new Vector2(4, 4);

    internal struct RayOrigins
    {
        public Vector2 topLeft, topRight, bottomLeft, bottomRight;
    }

    internal RayOrigins rayOrigins;
    internal Vector2 raySpacing;
    
    private BoxCollider2D _collider;

    // ----------------------------------------------------------------------------------------

    protected virtual void Awake()
    {
        _collider = GetComponent<BoxCollider2D>();
        GetComponent<Rigidbody2D>().isKinematic = true;

        CalculateRaySpacing();
    }

    // ----------------------------------------------------------------------------------------

    public void CalculateRaySpacing()
    {
        Bounds bounds = _collider.bounds;
        bounds.Expand(-2 * SkinWidth);

        raySpacing.x = bounds.size.y / (rayCount.x - 1);
        raySpacing.y = bounds.size.x / (rayCount.y - 1);
    }

    // ----------------------------------------------------------------------------------------

    public void UpdateRayOrigins()
    {
        Bounds bounds = _collider.bounds;
        bounds.Expand(-2 * SkinWidth);

        rayOrigins.topLeft = new Vector2(bounds.min.x, bounds.max.y);
        rayOrigins.topRight = new Vector2(bounds.max.x, bounds.max.y);
        rayOrigins.bottomLeft = new Vector2(bounds.min.x, bounds.min.y);
        rayOrigins.bottomRight = new Vector2(bounds.max.x, bounds.min.y);
    }

    // ----------------------------------------------------------------------------------------
}