﻿using UnityEngine;
using System.Collections;

[RequireComponent(typeof(Light))]
public class TorchFlicker : MonoBehaviour 
{
	//--- MEMBERS -----------------------------------------------------------------------------------------

	[Header("RangeSettings")]
	[SerializeField, Range(0f, 5f)] private float _minSpeed;
	[SerializeField, Range(0f, 5f)] private float _maxSpeed;
	[SerializeField, Range(0f, 20f)] private float _minRange, _maxRange;

	[Header("IntensitySettings")]
	[SerializeField, Range(1f, 5f)] private float _intensityMod = 1.2f;
	[SerializeField, Range(0, 100)] private int _offChance, _onChance;

	private Light _light;

	private float _sinTime;
	private int _direction = 1;
	private bool _intensityOff = false;

	//---- METHODS ----------------------------------------------------------------------------------------

	private void OnValidate()
	{
		if(_maxSpeed < _minSpeed)
			_maxSpeed = _minSpeed;

		if(_maxRange < _minRange)
			_maxRange = _minRange;
	}

	//-----------------------------------------------------------------------------------------------------

	private void Start() 
	{
		_light = GetComponent<Light>();
	}

	//-----------------------------------------------------------------------------------------------------

	private void Update() 
	{
		
		_sinTime += Time.deltaTime * Random.Range(_minSpeed, _maxSpeed) * _direction;
		if(_sinTime >= 1f || _sinTime <= 0f)
			_direction *= -1;

		_sinTime = Mathf.Clamp(_sinTime, 0f, 1f);
		_light.range = Mathf.Lerp(_minRange, _maxRange, _sinTime);

		int chance = Random.Range(0, 101);
		if(_intensityOff == false && chance < _offChance)
		{			
			_light.intensity /= _intensityMod;
			_intensityOff = true;
						
		}
		else if(_intensityOff == true && chance < _onChance)
		{			
			_light.intensity *= _intensityMod;
			_intensityOff = false;
		}
	}

	//-----------------------------------------------------------------------------------------------------
}