﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

[RequireComponent(typeof(BoxCollider2D), typeof(Animator))]
public class Wind : MonoBehaviour
{
    public const float Constantly = 0;
    [Range(0, 5)] public float interval = 1f;

	private enum WindDirection{Vertical, Horizontal, Diagonal}
	[SerializeField] private WindDirection _windDirection;

    private Animator _animator;
    private bool _containsPlayer, _containsRock;

    // ----------------------------------------------------------------------------------------

    public void Awake()
    {
        _animator = GetComponent<Animator>();
        _containsPlayer = false;
		_containsRock = false;                
    }

	// ----------------------------------------------------------------------------------------

	private void Start()
	{
		StartCoroutine(BlowInInterval());
	}

    // ----------------------------------------------------------------------------------------

    private IEnumerator BlowInInterval()
    {
        yield return new WaitForSeconds(interval);
        _animator.SetTrigger("Blow");
		if(_containsRock && _windDirection == WindDirection.Diagonal)
			yield break;

		//while(_animator.IsInTransition(0) == true)
		//	yield return null;

        float animationLength = _animator.GetCurrentAnimatorStateInfo(0).length;
        float stamp = Time.time;

        while (Time.time < stamp + animationLength)
        {
			if(_containsRock && _windDirection == WindDirection.Diagonal)
				yield break;

            if (_containsPlayer)
            {
                GameManager.instance.KillPlayer(GameManager.PlayerKillMethod.Torch);
				yield break;
            }
            yield return null;
        }

        StartCoroutine(BlowInInterval());
    }

    // ----------------------------------------------------------------------------------------

    private void OnTriggerEnter2D(Collider2D other)
    {
		if(enabled == false)
			return;
		
		if(_containsRock && _windDirection == WindDirection.Diagonal)
			return;

		if (other.gameObject.CompareTag("Rock"))
		{
			_containsRock = true;
		}
        else if (other.gameObject.CompareTag("Caveman"))
        {
            if(interval == Constantly)
            {
                GameManager.instance.KillPlayer(GameManager.PlayerKillMethod.Torch);
                return;
            }

            _containsPlayer = true;
        }
    }

    // ----------------------------------------------------------------------------------------

    private void OnTriggerExit2D(Collider2D other)
    {
		if (other.gameObject.CompareTag("Rock"))
		{
			if (_windDirection == WindDirection.Vertical && transform.position.y > other.transform.position.y ) 
			{
				Destroy (other.gameObject);
				Destroy (gameObject);
			} 
			else if(_windDirection == WindDirection.Diagonal)
			{
				_containsRock = false;
				StartCoroutine(BlowInInterval());
			}
		}
        else if (other.gameObject.CompareTag("Caveman"))
        {
            _containsPlayer = false;
        }
    }

    // ----------------------------------------------------------------------------------------
}